"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.buildCommand = exports.buildOption = exports.registerLongName = exports.isOptionAlreadyRegistered = exports.getShortFlagForLongName = exports.createCommand = void 0;
const commander_1 = require("commander");
const types_1 = require("../types");
const root_1 = require("../commands/handler/root");
const logger_1 = require("../logger");
const registeredOptions = {};
const getLogger = () => (0, logger_1.get)("utils.commands");
const checkEmpty = (name, value) => {
    if (value.trim() === "") {
        throw new Error(`missing value for parameter ${name}`);
    }
};
const checkCommand = (command) => {
    const { command: name, description } = command;
    try {
        checkEmpty("command", name);
        checkEmpty("description", description);
    }
    catch (err) {
        const { trace } = getLogger();
        trace(`failed to check command ${JSON.stringify(command, null, 2)} because of`, err);
        throw err;
    }
};
const createCommand = (name) => {
    const { output } = getLogger();
    const command = new commander_1.Command(name);
    command.enablePositionalOptions();
    command.exitOverride();
    command.configureOutput({ writeErr: output, writeOut: output });
    command.allowUnknownOption();
    return command;
};
exports.createCommand = createCommand;
const getShortFlagForLongName = (longName) => {
    for (const s of Object.keys(registeredOptions)) {
        if (registeredOptions[s] === longName) {
            return s;
        }
    }
    throw new Error(`no short flag registered for long name ${longName}`);
};
exports.getShortFlagForLongName = getShortFlagForLongName;
const isOptionAlreadyRegistered = ({ longName }, command) => {
    const opts = command.options?.map((o) => o.long?.replace("--", ""));
    return !!opts?.includes(longName);
};
exports.isOptionAlreadyRegistered = isOptionAlreadyRegistered;
const registerLongName = (longName) => {
    try {
        return (0, exports.getShortFlagForLongName)(longName);
    }
    catch (err) {
        for (const c of [
            longName[0],
            longName[0].toUpperCase() === longName[0]
                ? longName[0].toLowerCase()
                : longName[0].toUpperCase(),
            ...types_1.CHARACTERS,
        ]) {
            if (!registeredOptions[c]) {
                registeredOptions[c] = longName;
                return c;
            }
        }
        throw new Error(`no short flag found for name ${longName}`);
    }
};
exports.registerLongName = registerLongName;
const buildOption = async ({ longName, description, choices, required, default: defaultValue, args, hidden, }) => {
    const mandatory = !!required;
    const shortFlag = (0, exports.registerLongName)(longName);
    let newDescr = description;
    if (!mandatory) {
        newDescr = newDescr ? `${newDescr} (optional)` : "(optional)";
    }
    const argStr = args
        ? args.reduce((p, c) => (c.optional ? `${p} [${c.name}]` : `${p} <${c.name}>`), "")
        : "";
    const option = new commander_1.Option(`-${shortFlag}, --${longName}${argStr}`, newDescr)
        .makeOptionMandatory(mandatory)
        .hideHelp(!!hidden);
    if (defaultValue) {
        const defaultStr = typeof defaultValue === "function" ? await defaultValue() : defaultValue;
        option.default(defaultStr);
    }
    let choicesArr = [];
    if (choices) {
        choicesArr = typeof choices === "function" ? await choices() : choices;
    }
    if (choicesArr.length > 0) {
        option.choices(choicesArr);
    }
    return option;
};
exports.buildOption = buildOption;
const buildCommand = async (topCommand, command) => {
    []
        .concat([command], command.type === "topCommand" ? command.subCommands : [])
        .forEach((c) => checkCommand(c));
    const { command: name, description, options } = command;
    const comm = (0, exports.createCommand)(name);
    comm.description(description);
    comm.enablePositionalOptions();
    for (const option of options ?? []) {
        // eslint-disable-next-line no-await-in-loop
        comm.addOption(await (0, exports.buildOption)(option));
    }
    if (command.type === "topCommand") {
        const { subCommands } = command;
        for (const subCommand of subCommands) {
            // eslint-disable-next-line no-await-in-loop
            await (0, exports.buildCommand)(comm, subCommand);
        }
    }
    else {
        // command.type === "command"
        const { args, handler } = command;
        if (args) {
            args.forEach((a) => comm.argument(`<${a.argument}>`, a.description));
        }
        const rootHandler = await (0, root_1.create)(handler, `Failed to ${command.description}`)(comm);
        comm.action(rootHandler);
    }
    topCommand.addCommand(comm);
    return comm;
};
exports.buildCommand = buildCommand;
